
/**************************************************************************
*   Coraid Monitoring Tool is a interface to communication ports in java
* 	for Coraid products.
*   Copyright 2006 by Alyseo info@alyseo.com
*
*   This library is free software; you can redistribute it and/or
*   modify it under the terms of the GNU Library General Public
*   License as published by the Free Software Foundation; either
*   version 2 of the License, or (at your option) any later version.
*
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*   Library General Public License for more details.
*
*   You should have received a copy of the GNU Library General Public
*   License along with this library; if not, write to the Free
*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *****************************************************************************/

package com.alyseo.serial;

import gnu.io.SerialPort;

import java.util.ArrayList;
import java.util.regex.*;

import org.apache.log4j.Logger;

public class CoraidConf {

	public enum NagiosCodeEnum {OK, WARNING, CRITICAL, UNKNOWN }; 
	private String bps;
	private String data;
	private String parity;
	private String stopBits;
	private String coraidPwd;
	private String nagiosHostname;
	private String shelfName;
	private String commPortName;
	private String nagiosServiceDesc;
	private ArrayList<String> mesgs = new ArrayList<String>();
	private CoraidMsgStruct coraidMsgStruct = new CoraidMsgStruct();
	private NagiosCodeEnum nagiosCode = NagiosCodeEnum.WARNING;
	private String nagiosMessage;
	
	private Logger log = Logger.getLogger("CoraidConf");

	public CoraidConf(String commPortName, String conf) {
		this.commPortName = commPortName;
		// COM1=                            9600  ,  8  ,  N  ,  1  ,Alyseo,SR production,Coraid SR1520,check failed disk
		Pattern pattern = Pattern.compile("^(\\d+),(\\d),(\\w),(\\d),(.+?),(.*?),(.+?),(.*)$");
		Matcher matcher = pattern.matcher(conf);
		boolean result = matcher.matches();
		if(result) {
			bps = matcher.group(1);
			data = matcher.group(2);
			parity = matcher.group(3);
			stopBits = matcher.group(4);
			coraidPwd = matcher.group(5);
			shelfName = matcher.group(6);
			nagiosHostname = matcher.group(7);
			nagiosServiceDesc = matcher.group(8);
		}
	}

	public int getBps() {
		return Integer.parseInt(bps);
	}
	public int getDataSize() {
		return (Integer.parseInt(data) == 5) ? SerialPort.DATABITS_5 :
			(Integer.parseInt(data) == 6) ? SerialPort.DATABITS_6 :
			(Integer.parseInt(data) == 7) ? SerialPort.DATABITS_7 : SerialPort.DATABITS_8;
	}
	public int getParity() {
		return parity.equals("O") ? SerialPort.PARITY_ODD :
			parity.equals("E") ? SerialPort.PARITY_EVEN : 
			SerialPort.PARITY_NONE;
	}
	public int getStopBits() {
		return Integer.parseInt(stopBits) == 1 ? SerialPort.STOPBITS_1 : SerialPort.STOPBITS_2;
	}
	public String getCoraidPwd() {
		return coraidPwd;
	}
	public String getShelfName() {
		return shelfName;
	}
	public String getPortName() {
		return commPortName;
	}
	public String getNagiosServiceDesc() {
		return nagiosServiceDesc;
	}
	
	public String toString() {
		return "CoraidConf of " + commPortName + " = " + bps+"/"+data+"/"+parity+"/"+stopBits+"/"+
		coraidPwd+"/"+shelfName+"/"+"/"+nagiosServiceDesc;
	}
	public void addCoraidConf(String conf) {
		coraidMsgStruct.addConf(conf);
	}
	public void	addWhenEntry(String conf){
		coraidMsgStruct.addWhenEntry(conf);
	}
	public void printConf() {
		coraidMsgStruct.print();
	}
	public void analyseConf() {
		setNagiosCode(coraidMsgStruct.getNagiosCode());
		addNagiosMessage(coraidMsgStruct.getNagiosMesg());
	}
	public void addNagiosMessage(String nagiosMesg) {
		this.nagiosMessage = nagiosMesg;
	}
	
	public void clearMessages() {
		coraidMsgStruct.clear();
	}
	public String getNagiosMessage() {
		return nagiosMessage;
	}
	public void setNagiosCode(NagiosCodeEnum code) {
		this.nagiosCode = code;
	}
	public String getNagiosCode() {
		switch (nagiosCode) {
			case OK : return "0";
			case WARNING : return "1";
			case CRITICAL : return "2";
			default : return "3";
		}
	}

	public String getNagiosHostname() {
		return nagiosHostname;
	}

	public void setShelfName(String shelfName) {
		if(shelfName.equals("")) {
			this.shelfName = shelfName;
		}
	}
}
