
/**************************************************************************
*   Coraid Monitoring Tool is a interface to communication ports in java
* 	for Coraid products.
*   Copyright 2006 by Alyseo info@alyseo.com
*
*   This library is free software; you can redistribute it and/or
*   modify it under the terms of the GNU Library General Public
*   License as published by the Free Software Foundation; either
*   version 2 of the License, or (at your option) any later version.
*
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*   Library General Public License for more details.
*
*   You should have received a copy of the GNU Library General Public
*   License along with this library; if not, write to the Free
*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *****************************************************************************/

package com.alyseo.serial;

import gnu.io.*;

import java.io.*;
import java.util.*;
import java.util.regex.*;


import static com.alyseo.serial.CoraidConf.*;

import org.apache.log4j.*;

public class CoraidSerialPort extends TimerTask implements SerialPortEventListener,Runnable {
	
	private CoraidConf coraidConf;
	private String shelfName;
	private CommPortIdentifier portIdentifier;
	private CommPort commPort;
	private SerialPort serialPort;
	private InputStream in;
	private OutputStream out;
	private PrintWriter pw;
	private StringBuffer buffer = new StringBuffer();
	private Logger log = Logger.getLogger("CoraidSerialPort");

	public CoraidSerialPort (CoraidConf coraidConf) {
		this.coraidConf = coraidConf;
		log.debug(coraidConf);
	}

	private boolean init () {
		boolean result = true;

		try {
			portIdentifier = CommPortIdentifier.getPortIdentifier(coraidConf.getPortName());
			log.debug("Comm port owned : " + portIdentifier.isCurrentlyOwned()); 

			commPort = portIdentifier.open(null,0);
            serialPort = (SerialPort) commPort;
    		serialPort.addEventListener(this);
    		serialPort.notifyOnDataAvailable(true);
			serialPort.setSerialPortParams(coraidConf.getBps(),
					coraidConf.getDataSize(),
					coraidConf.getStopBits(),
					coraidConf.getParity());
//			serialPort.setFlowControlMode(SerialPort.FLOWCONTROL_RTSCTS_IN|SerialPort.FLOWCONTROL_RTSCTS_OUT);
			serialPort.setFlowControlMode(SerialPort.FLOWCONTROL_XONXOFF_IN | SerialPort.FLOWCONTROL_XONXOFF_OUT);
			serialPort.setInputBufferSize(2048);
			serialPort.setOutputBufferSize(1024);

		   	in = serialPort.getInputStream();
	        out = serialPort.getOutputStream();
	        pw = new PrintWriter(out);

		} catch (NoSuchPortException ex) {
			log.fatal("No such port : " + coraidConf.getPortName());
			result = false;
		} catch (PortInUseException ex) {
			coraidConf.setNagiosCode(NagiosCodeEnum.WARNING);
			coraidConf.clearMessages();
			coraidConf.addNagiosMessage("Serial port " + coraidConf.getPortName() + " is already in use by another process");
			EventsFifo.getInstance().addMessage(coraidConf);
			log.fatal("Unable to access port " + coraidConf.getPortName() + ", port is already in use by another process.");
			result = false;
		} catch (UnsupportedCommOperationException ex) {
			coraidConf.setNagiosCode(NagiosCodeEnum.WARNING);
			coraidConf.clearMessages();
			coraidConf.addNagiosMessage("Unable to configure port " + coraidConf.getPortName() + " with settings '" + serialPort.toString());
			EventsFifo.getInstance().addMessage(coraidConf);
			log.fatal("Unable to configure port " + coraidConf.getPortName() + " with settings '" + serialPort.toString());
			result = false;
		} catch (IOException ex) {
			coraidConf.setNagiosCode(NagiosCodeEnum.WARNING);
			coraidConf.clearMessages();
			coraidConf.addNagiosMessage("Unable to get I/O stream on port " + coraidConf.getPortName());
			EventsFifo.getInstance().addMessage(coraidConf);
			log.fatal("Unable to get I/O stream on port " + coraidConf.getPortName() );
			result = false;
		} catch (TooManyListenersException e) {
			log.fatal("Unable to listen port : " + coraidConf.getPortName() + ". Too many listeners");
			result = false;
		} catch (Exception e) {
			log.fatal("Error in init function : " + e.getMessage());
		} 
		
        return result;
	}

	public void serialEvent(SerialPortEvent evt) {
		switch (evt.getEventType() ) {
			case SerialPortEvent.DATA_AVAILABLE:
				byte[] readBuffer = new byte[512];
				int nByte = 0;
				try {
					while (in.available() > 0)	{
						nByte = in.read(readBuffer);
						String b = new String(readBuffer,0,nByte);
						buffer.append(b);
					}
				} catch (IOException e) { 
					log.error("Can not read input stream from " + coraidConf.getPortName());
				}
				break;
			default:
				break;				
		}
	}

	private void checkWhen() throws NoDataException{
		send("when\n");
		String r = waitData();

		String[] lines = Pattern.compile("\n", Pattern.MULTILINE).split(r);
		for(int i = 0; i < lines.length; i ++) {
			coraidConf.addWhenEntry(lines[i]);
		}				

		
	}

	private void checkDisk() throws NoDataException {
		send("list -l\n");
		String r = waitData();
		String[] lines = Pattern.compile("\n", Pattern.MULTILINE).split(r);
		coraidConf.clearMessages();
		for(int i = 0; i < lines.length; i ++) {
			coraidConf.addCoraidConf(lines[i]);
		}
		checkWhen();
		coraidConf.analyseConf();
		coraidConf.printConf();
		EventsFifo.getInstance().addMessage(coraidConf);
	}
	
	private String waitData () throws NoDataException  {
   	    Pattern p = Pattern.compile("^SR shelf (\\d+)> $", Pattern.MULTILINE);
		log.debug("Waiting for answer from " + coraidConf.getPortName());
 	    
		Calendar c =  new GregorianCalendar();
		boolean found = false;

		for(int tryMatch=1; tryMatch < 4; tryMatch ++) {
   	    	Matcher m = p.matcher(buffer.toString());
   	    	if( m.find()) {
   	    		found = true;
   	    		break;
   	    	}
	    	try {
	    		log.debug("Did not match regexp ! waiting for " + tryMatch + "s");
				Thread.currentThread().sleep(tryMatch * 1000);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
		log.debug("Answer from " + coraidConf.getPortName() + " : (" + buffer.toString() + ")");
		if(found) {
			return buffer.toString();
		} else {
			coraidConf.setNagiosCode(NagiosCodeEnum.CRITICAL);
			coraidConf.clearMessages();
			coraidConf.addNagiosMessage("No data received from serial port");
			log.fatal("No data received from serial port");
			EventsFifo.getInstance().addMessage(coraidConf);
			throw new NoDataException("No data received from serial port");
		}
	}
	
	private void checkShelfName() throws NoDataException {
		send("shelf\n");
		String r = waitData();
		Pattern p = Pattern.compile("^(\\d+)\\s*$", Pattern.MULTILINE);
		Matcher m = p.matcher(r);
		if(m.find()) {
			shelfName = m.group(1);
			log.debug("Detecting shelf name : "+ shelfName);
			coraidConf.setShelfName(shelfName);
		} else {
			log.error("Unable to detect Shelf name");
		}
	}

	private void checkPassword() throws NoDataException {

		send("\n");
		
		Calendar c =  new GregorianCalendar();
		boolean found = false;
		log.debug("Waiting for answer from " + coraidConf.getPortName());
		Pattern p = Pattern.compile(".*(password:|SR shelf (\\d+)>)\\s$", Pattern.DOTALL);
		
		for(int tryMatch=1; tryMatch < 4; tryMatch ++) {
			Matcher seq = p.matcher(buffer.toString());
			if( seq.matches()) {
				found = true;
				break;
			}
	    	try {
				Thread.currentThread().sleep(tryMatch * 1000);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
		log.debug("Answer from " + coraidConf.getPortName() + " : (" + buffer.toString() + ")");
		if(found) {
			if(buffer.toString().contains("password")) {
				log.debug("Log on coraid box, sending password..." );
				send(coraidConf.getCoraidPwd() + "\n");
			} else {
				log.warn("Someone has already logged on coraid box ...");
			}
		} else {
			coraidConf.setNagiosCode(NagiosCodeEnum.WARNING);
			coraidConf.clearMessages();
			log.warn("Unable to get data from serial port");
			coraidConf.addNagiosMessage("No data received from serial port");
			EventsFifo.getInstance().addMessage(coraidConf);
			throw new NoDataException("No data received from serial port");
		}
	}

	public void send(String msg) {
		log.debug("sending \"" + msg.replaceAll("\n", "\\\\n") + "\" to " + coraidConf.getPortName());
		buffer = new StringBuffer();

		pw.print(msg);
		pw.flush();
		try {
			Thread.currentThread().sleep(1000);
		} catch (InterruptedException e) {
			e.printStackTrace();
			log.error(e.getMessage());
		}

	}

	public void closeAll() throws IOException{
		send("exit\n");
		pw.close();
		in.close();
		out.close();
		serialPort.removeEventListener();
		serialPort.close();
		commPort.close();
	}

	
	public void run() {
		log.info("Start checking thread ...");
		boolean result;
		result = init();

		if(result) {
			try {
				checkPassword();
				checkShelfName();
				checkDisk();
			} catch (NoDataException ex) {
				log.error(ex.getMessage());
			} finally {
				try {
					closeAll();
				} catch (IOException ex) {
					log.error("Can not close all serial streams : " + ex.getMessage());
				}
			}
		}
		
		log.debug("ending monitoring thread");
	}
}
